@@ -163,8 +163,8 @@ def get_existing_keywords(googleads_client,customer_id):
           ad_group_criterion.criterion_id,
           ad_group_criterion.keyword.text,
           ad_group_criterion.keyword.match_type
-        FROM ad_group_criterion
-        WHERE ad_group_criterion.type = KEYWORD"""
+        FROM ad_group_criterion       
+        WHERE ad_group_criterion.type = KEYWORD AND campaign.status IN ('ENABLED', 'PAUSED')"""
 
     search_request = googleads_client.get_type("SearchGoogleAdsRequest")
     search_request.customer_id = customer_id
@@ -193,7 +193,7 @@ def get_existing_keywords(googleads_client,customer_id):
     df = pd.DataFrame(camps)
     return df
 
-def create_keyword(googleads_client,customer_id,ad_group_id, keyword_text, kw_type):
+def create_keyword(googleads_client,customer_id,ad_group_id, keyword_text, kw_type,negative=False):
     ad_group_service = googleads_client.get_service("AdGroupService")
     ad_group_criterion_service = googleads_client.get_service(
         "AdGroupCriterionService")
@@ -216,6 +216,7 @@ def create_keyword(googleads_client,customer_id,ad_group_id, keyword_text, kw_ty
     ad_group_criterion.keyword.match_type = (
         kw_type
     )
+    ad_group_criterion.negative = negative
 
     # Optional field
     # All fields can be referenced from the protos directly.
@@ -337,6 +338,72 @@ def create_campaign(googleads_client,customer_id,campaignName, budgetName, budge
     except GoogleAdsException as ex:
         _handle_googleads_exception(ex)
 
+def add_negative_keywords(client,customer_id,shared_set_id,keywords,kw_type):
+    shared_set_service = client.get_service("SharedSetService")
+    shared_criterion_service = client.get_service("SharedCriterionService")
+
+    shared_criterion_operations = list()
+
+    for keyword in keywords:
+        shared_criterion_operation = client.get_type("SharedCriterionOperation")
+        shared_criterion = shared_criterion_operation.create
+        shared_criterion.shared_set = shared_set_service.shared_set_path(customer_id, shared_set_id)
+        shared_criterion.keyword.text = keyword
+        if kw_type == 'PHRASE':
+            kw_type = client.enums.KeywordMatchTypeEnum.PHRASE
+        elif kw_type == 'BROAD':
+            kw_type = client.enums.KeywordMatchTypeEnum.BROAD
+        else:
+            kw_type = client.enums.KeywordMatchTypeEnum.EXACT
+        
+        shared_criterion.keyword.match_type = kw_type
+
+        shared_criterion_operations.append(shared_criterion_operation)
+
+    try:
+        shared_set_criterion_response = (
+            shared_criterion_service.mutate_shared_criteria(
+                customer_id=customer_id,
+                operations=shared_criterion_operations,
+            )
+        )
+        for response in shared_set_criterion_response.results:
+            print(
+                "Created Shared Set Criteria"
+                f"{response.resource_name}."
+                )
+
+        return shared_set_criterion_response.results
+    except GoogleAdsException as ex:
+        _handle_googleads_exception(ex)
+
+def get_all_shared_sets(client,customer_id):
+    df_dict = {
+        "shared_set_id": list(),
+        "shared_set_name": list(),
+    }
+    query = """
+    SELECT 
+    shared_set.id, 
+    shared_set.name 
+    FROM shared_set 
+    """
+
+    ga_service = client.get_service("GoogleAdsService")
+    search_request = client.get_type("SearchGoogleAdsStreamRequest")
+    search_request.customer_id = customer_id
+    search_request.query = query
+    stream = ga_service.search_stream(search_request)
+    
+    for batch in stream:
+        for row in batch.results:
+            df_dict['shared_set_id'].append(row.shared_set.id)
+            df_dict['shared_set_name'].append(row.shared_set.name)
+    
+    df = pd.DataFrame(df_dict)
+    return df
+
+
 def get_all_ads(client,customer_id):
     query = """
     SELECT
@@ -538,6 +605,21 @@ def remove_adgroup(client, customer_id, ad_group_id):
 
     print(f"Removed ad group {ad_group_response.results[0].resource_name}.")
 
+def remove_keyword(client, customer_id, ad_group_id, criterion_id):
+    agc_service = client.get_service("AdGroupCriterionService")
+    agc_operation = client.get_type("AdGroupCriterionOperation")
+
+    resource_name = agc_service.ad_group_criterion_path(
+        customer_id, ad_group_id, criterion_id
+    )
+    agc_operation.remove = resource_name
+
+    agc_response = agc_service.mutate_ad_group_criteria(
+        customer_id=customer_id, operations=[agc_operation]
+    )
+
+    print(f"Removed keyword {agc_response.results[0].resource_name}.")
+
 
 def get_search_term_report(client, customer_id,start_date='2022-01-01', end_date='2022-01-10'):
     df_dict = {
@@ -553,6 +635,8 @@ def get_search_term_report(client, customer_id,start_date='2022-01-01', end_date
     'adgroup_id': list(),
     'adgroup_status': list(),
     'adgroup_camp': list(),
+    'campaign_id': list(),
+    'campaign_status': list(),
     'metrics_clicks': list(),
     'metrics_conversion_value' : list(),
     'metrics_conversions' : list(),
@@ -595,10 +679,13 @@ def get_search_term_report(client, customer_id,start_date='2022-01-01', end_date
             metrics.all_conversions, 
             ad_group.id, 
             ad_group.status,
-            ad_group.campaign
+            ad_group.campaign,
+            campaign.id,
+            campaign.status
         FROM search_term_view
+        WHERE campaign.status IN ('ENABLED','PAUSED') AND
         """
-        query2 = f"WHERE segments.date >= '{str(start)[0:10]}' AND segments.date < '{str(start+step)[0:10]}'"
+        query2 = f" segments.date >= '{str(start)[0:10]}' AND segments.date < '{str(start+step)[0:10]}'"
         query = query1 + '\n' + query2
         search_request = client.get_type("SearchGoogleAdsStreamRequest")
         search_request.customer_id = customer_id
@@ -625,6 +712,9 @@ def get_search_term_report(client, customer_id,start_date='2022-01-01', end_date
                 df_dict['adgroup_status'].append(adgroup.status.name)
                 df_dict['adgroup_id'].append(adgroup.id)
                 df_dict['adgroup_camp'].append(adgroup.campaign)
+
+                df_dict['campaign_id'].append(row.campaign.id)
+                df_dict['campaign_status'].append(row.campaign.status.name)
                 
                 df_dict['metrics_clicks'].append(metrics.clicks)
                 df_dict['metrics_conversion_value'].append(metrics.conversions_value)
